# This file was auto-generated by Fern from our API Definition.

import typing

from .bearer_auth import BearerAuth
from .header_auth import HeaderAuth

RemoteAuth = typing.Union[BearerAuth, HeaderAuth]
