# This file was auto-generated by Fern from our API Definition.

import typing

from .max_chars_chunking_strategy import MaxCharsChunkingStrategy
from .sentence_chunking_strategy import SentenceChunkingStrategy

ChunkingStrategy = typing.Union[MaxCharsChunkingStrategy, SentenceChunkingStrategy]
